/*
 * Klogwatch, Netfilter log monitor
 *
 * Copyright (C) 2004 Nick Battle <nick.battle@freeuk.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <kmessagebox.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <unistd.h>
#include <stdlib.h>
#include <qbuttongroup.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qcursor.h>
#include "klogconfig.h"
#include "kloglist.h"
#include "klogitem.h"
#include "klogpid.h"
#include "klogIP.h"
#include "debug.h"

KLogList::KLogList(QString file, QWidget *parent): QTlist(parent)
{
    setFocusPolicy(QWidget::StrongFocus);

    QFontMetrics fm(klogconfig->FONT);
    listbox->setFont(klogconfig->FONT);
    int fh = fm.height();
    debug("Font height = %d\n", fh);
    int im = listbox->itemMargin();
    debug("Item margin = %d\n", im);

    // list lines, plus header, plus bevel margins
    int height = (int)(((fh + 2*im) * klogconfig->MINLINES) + (fh + 6) + 6);
    debug("Window height = %d\n", height);
    listbox->setMinimumHeight(height);

    int w, width = 0;
    listbox->setColumnWidthMode(0, QListView::Manual);
    listbox->setColumnWidth(0, w = fm.width("88:88:88")+10);
    width += w;
    listbox->setColumnWidthMode(1, QListView::Manual);
    listbox->setColumnWidth(1, w = fm.width("Protocol")+10);
    width += w;
    listbox->setColumnWidthMode(2, QListView::Manual);
    listbox->setColumnWidth(2, w = fm.width("888.888.888.888")+10);
    width += w;
    listbox->setColumnWidthMode(3, QListView::Manual);
    listbox->setColumnWidth(3,
    		w = fm.width("destination-unreachable(3), code 8")+10);
    width += w;
    listbox->setColumnWidthMode(4, QListView::Manual);
    listbox->setColumnWidth(4, w = fm.width("XYZ XYZ XYZ XYZ")+10);
    width += w + 5 + 15;	// + 1 per column, + scrollbar
    debug("Window width = %d\n", width);
    listbox->setMinimumWidth(width);

    listbox->setSorting(-1, false);
    listbox->setAllColumnsShowFocus(true);
    listbox->resize(width, height);

    connect(listbox, SIGNAL(doubleClicked(QListViewItem *)),
            this, SLOT(slotOpen(QListViewItem *)));
    connect(listbox, SIGNAL(pressed(QListViewItem *)),
	    this, SLOT(slotClick(QListViewItem *)));

    connect(hideButton, SIGNAL(clicked()), this, SLOT(slotHide()));
    connect(pauseButton, SIGNAL(clicked()), this, SLOT(slotPause()));
    connect(clearButton, SIGNAL(clicked()), this, SLOT(slotClear()));

    pauseButton->setToggleButton(true);
    pauseButton->setOn(false);

    listbox->setSorting(0);
    totalPacketCount = 0;
    uncheckedPacketCount = 0;
    oldestPacket = 1;

    if (klogconfig->SYSTRAY == 1)
    {
	tray = new KLogTray((KLogWatch *)parent, this);
	connect(tray, SIGNAL(quitSelected()), parent, SLOT(slotExit()));
    }
    else
    {
	tray = NULL;
    }

    tailer = new KLogTail(file.latin1());
    state = (debugOn ? THINKING : RUNNING);
    timer = new QTimer(this, "timer");
    timer->start(1000);
    connect(timer, SIGNAL(timeout()), this, SLOT(slotTimer()));

    pollCount = 0;
    showCount = 0;
    pauseCount = 0;
}

KLogList::~KLogList()
{
    debug("Destroy KLogList\n");
}

void KLogList::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
	case Key_Enter:
	case Key_Return:
	    debug("Got Enter/Return\n");
	    slotHide();
	    break;

	default:
	    debug("Got key, Thinking...\n");
	    thinking();
	    hideButton->setText("Hide");
	    break;
    }
}

bool KLogList::addNew()
{
    if (++pollCount < klogconfig->POLLTIME)
    {
	return false;	// Not time to look yet
    }

    pollCount = 0;
    bool added = false;
    mutex.lock();

    for (int count = 0; count < MAXNEWROWS; count++)
    {
	const KLogLine *line = tailer->getLogLine();

	if (line == NULL)
	{
	    debug("."); fflush(stdout);
	    break;
	}

	debug("++++ packet ++++\n");

	if (line->isPacket())
	{
	    KLogItem *item;

            totalPacketCount++;
	    uncheckedPacketCount++;

	    item = new KLogItem(listbox, line);

	    item->isNew = true;
            item->packetNumber = totalPacketCount;

	    if (listbox->childCount() > klogconfig->BUFFSIZE)
	    {
                QListViewItemIterator it( listbox );
                while ( it.current() ) {

                    if ( ((KLogItem*)it.current())->packetNumber == oldestPacket) {
                        debug("Removing oldest item\n");
                        //listbox->takeItem(it.current());
                        delete(it.current());
                        oldestPacket++;
                        break;
                    }
                    ++it;
                }
	    }

	    if ( !(listbox->isVisible()) ) {
	        listbox->setCurrentItem(item);
	        listbox->ensureItemVisible(item);
	    }

	    char label[64];
	    sprintf(label, "%lu packets detected (%lu new)", totalPacketCount, uncheckedPacketCount);
	    packetCount->setText(label);

	    added = true;
	}
	else
	{
	    debug("Deleting non-packet line\n");
	    delete line;
	    count--;
	}
    }

    if (added)
    {
	emit sigShow(false);
	if (tray) tray->setTrayIcon(true);

	// This fixes the first time popup focus bug
	static bool first = true;

	if (first)
	{
	    sleep(1);
	    emit sigShow(false);
	    first = false;
	}
    }

    mutex.unlock();
    return added;
}

void KLogList::thinking()
{
    debug("Thinking\n");
    state = THINKING;
}

void KLogList::running()
{
    debug("Running\n");
    state = RUNNING;
}

void KLogList::markOldPackets() {
    debug("clearNewEntries\n");
    QListViewItemIterator it(listbox);
    QListViewItem *item;

    for (; (item = it.current()); it++)
    {
      ((KLogItem *)item)->isNew = false;
    }
    uncheckedPacketCount = 0;

    char label[64];
    sprintf(label, "%lu packets detected (%lu new)", totalPacketCount, uncheckedPacketCount);
    packetCount->setText(label);

    if (tray) tray->setTrayIcon(false);
}


void KLogList::slotTimer()
{
    if (KLogTerm())
    {
        slotExit();
	return;
    }

    if (KLogShow())
    {
        emit sigShow(true);	// force popup for signal
        thinking();
        showCount = 0;
    }

    if (pauseButton->state() == QButton::On)
    {
	if (++pauseCount > klogconfig->PAUSETIME)
	{
	    pauseButton->setText("Pause");
	    pauseButton->setOn(false);
	    pauseCount = 0;
	}
	else
	{
	    return;	// No processing while paused
	}
    }

    if (state == THINKING)
    {
	addNew();
    }
    else
    {
	if (addNew())
	{
	    showCount = 0;
	}

	if (isVisible())
	{
	    if ((klogconfig->SHOWTIME > 0) &&	// zero never hides
		(klogconfig->TRAYALERT == 0))	// not showing anyway
	    {
		if (++showCount > klogconfig->SHOWTIME)
		{
		    emit sigHide();
		    hideButton->setText("Hide");
		    showCount = 0;
		}
		else if (isVisible())
		{
		    char h[20];
		    sprintf(h, "Hide in %d",
			klogconfig->SHOWTIME - showCount + 1);
		    hideButton->setText(h);
		}
	    }
	}
	else
	{
	    hideButton->setText("Hide");
	    showCount = 0;
	}
    }

    return;
}

void KLogList::slotHide()
{
    emit sigHide();
    hideButton->setText("Hide");
    running();
    markOldPackets();
}

void KLogList::slotPause()
{
    debug("Pause state is %s\n",
    	pauseButton->state() == QButton::On ? "ON" : "OFF");

    if (pauseButton->state() == QButton::Off)
    {
	state = THINKING;
	pauseCount = 0;
	pauseButton->setText("Pause");

	int n = listbox->childCount();

	if (n == 0)
	{
	    packetCount->setText("No packets detected");
	}
	else
	{
	    char label[64];
	    sprintf(label, "%lu packets detected (%lu new)", totalPacketCount, uncheckedPacketCount);
	    packetCount->setText(label);
	}
    }
    else
    {
	pauseCount = 0;
	pauseButton->setText("Paused");
	hideButton->setText("Hide");
	packetCount->setText("Packet detection paused");
    }
}

void KLogList::slotClear()
{
    thinking();
    hideButton->setText("Hide");
    listbox->clear();
    uncheckedPacketCount = 0;
    totalPacketCount = 0;
    oldestPacket = 1;
    if(tray) tray->setTrayIcon(false);
    packetCount->setText("No packets detected");
}

void KLogList::slotClick(QListViewItem *item)
{
    debug("Clicked\n");
    hideButton->setText("Hide");
    thinking();
    setFocus();
    if (item != 0) {
      if ( ((KLogItem*)item)->isNew ) {
          ((KLogItem*)item)->isNew = false;
          uncheckedPacketCount--;
          char label[64];
          sprintf(label, "%lu packets detected (%lu new)", totalPacketCount, uncheckedPacketCount);
          packetCount->setText(label);
      }
      if(tray) {
          if(uncheckedPacketCount < 1)
              tray->setTrayIcon(false);
          else
              tray->setTrayIcon(true);
      }

    }

}

QString KLogList::process(const KLogIP *packet)
{
    QString output;

    static char DATE[16];
    static char TIME[16];
    static char PROTO[16];
    static char SRC[64];
    static char SPT[64];
    static char DST[64];
    static char DPT[64];
    static char FLAGS[32];
    static char LINE[1024];

    sprintf(DATE, "DATE=%s", packet->getDate().latin1());
    putenv(DATE);
    sprintf(TIME, "TIME=%s", packet->getTime().latin1());
    putenv(TIME);
    sprintf(PROTO, "PROTO=%s", packet->getProto().latin1());
    putenv(PROTO);
    sprintf(SRC, "SRC=%s", packet->getSrcIP().latin1());
    putenv(SRC);
    sprintf(SPT, "SPT=%s", packet->getSrcPort().latin1());
    putenv(SPT);
    sprintf(DST, "DST=%s", packet->getDstIP().latin1());
    putenv(DST);
    sprintf(DPT, "DPT=%s", packet->getDstPort().latin1());
    putenv(DPT);
    sprintf(FLAGS, "FLAGS=%s", packet->getFlags().latin1());
    putenv(FLAGS);
    sprintf(LINE, "LINE=%s", packet->getLine());
    putenv(LINE);

    debug("Executing <%s>\n", klogconfig->COMMAND.latin1());
    FILE *f = popen(klogconfig->COMMAND.latin1(), "r");

    if (f == NULL)
    {
	output = "Command failed";
    }
    else
    {
	while (!feof(f))
	{
	    char b[256];

	    if (fgets(b, sizeof(b), f))
	    {
		debug("%s", b);
		output.append(b);
	    }
	}
    }

    if (output.isEmpty())
    {
	output = "No output";
    }

    pclose(f);
    return output;
}

void KLogList::slotOpen(QListViewItem *item)
{
    mutex.lock();
    debug("Double clicked\n");
    thinking();
    const KLogIP *packet = ((KLogItem *)item)->getPacket();
    setCursor(QCursor(waitCursor));
    QString q = process(packet);
    setCursor(QCursor());
    mutex.unlock();
    
    KMessageBox::information(this, q);
    setFocus();
}

void KLogList::saveTo(QTextStream &s)
{
    mutex.lock();
    thinking();

    QListViewItemIterator it(listbox);
    QListViewItem *item;

    for (; (item = it.current()); it++)
    {
        const KLogIP *packet = ((KLogItem *)item)->getPacket();
        s << packet->toString(true) << "\n";
    }

    mutex.unlock();
    return;
}

void KLogList::printTo(KPrinter &print)
{
    int x = 0, y = 20;
    QPainter paint;
    thinking();

    paint.begin(&print);
    paint.setFont(klogconfig->FONT);

    QPaintDeviceMetrics pm(&print);
    int maxy = pm.height();
    debug("max y offset = %d\n", maxy);
    QFontMetrics fm = paint.fontMetrics();
    int fh = fm.height();
    debug("font height = %d\n", fh);
    maxy -= fh;   // make sure we print last line!

    mutex.lock();
    QListViewItemIterator it(listbox);
    QListViewItem *item;

    for (; (item = it.current()); it++)
    {
        const KLogIP *packet = ((KLogItem *)item)->getPacket();
        QString text = packet->toString(true);
        paint.drawText(x, y, text);
        y += fh;

        if (y > maxy)
        {
            y = 20;
            print.newPage();
	}
    }

    mutex.unlock();
    paint.end();

    return;
}

void KLogList::slotExit()
{
    emit sigExit();
}

// eof
