/*
 * Klogwatch, Netfilter log monitor
 *
 * Copyright (C) 2004 Nick Battle <nick.battle@freeuk.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <khelpmenu.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kuniqueapplication.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <qfiledialog.h>
#include <kprinter.h>
#include <qwidgetlist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <unistd.h>
#include <errno.h>
#include "klogwatch.h"
#include "klogsetup.h"
#include "debug.h"

KLogWatch::KLogWatch(const char *logfile) :
    KMainWindow(0, 0,
    WType_TopLevel | WDestructiveClose | WStyle_ContextHelp)
{
    file = logfile;
    kloglist = new KLogList(file, this);
    setCentralWidget(kloglist);

    connect(kloglist, SIGNAL(sigHide()), this, SLOT(slotHide()));
    connect(kloglist, SIGNAL(sigShow(bool)), this, SLOT(slotShow(bool)));
    connect(kloglist, SIGNAL(sigExit()), this, SLOT(slotExit()));

    QString caption("Netfilter Log Events from ");
    setCaption(caption + file);
    KIconLoader *i = KGlobal::iconLoader();
    menubar = menuBar();
    QPopupMenu *filePopup = new QPopupMenu();

    filePopup->insertItem(i->loadIcon("filesave", KIcon::Small),
                "&Save", this, SLOT(slotSave()), CTRL+Key_S);

    filePopup->insertItem(i->loadIcon("configure", KIcon::Small),
                "S&etup", this, SLOT(slotSetup()), CTRL+Key_C);

    filePopup->insertItem(i->loadIcon("printer1", KIcon::Small),
                "&Print", this, SLOT(slotPrint()), CTRL+Key_P);

    filePopup->insertItem(i->loadIcon("exit", KIcon::Small),
                 "E&xit", this, SLOT(slotExit()), Key_Escape);

    menubar->insertItem("&File", filePopup);

    KAboutData *aboutData = new KAboutData(
        "klogwatch",
        I18N_NOOP("KLogWatch"),
        VERSION,
        I18N_NOOP("A simple Netfilter log file monitor"),
        KAboutData::License_GPL,
        "(c) 2004, Nick Battle",
        I18N_NOOP(
            "Klogwatch is a monitor for displaying packets logged to\n"
            "a file by the Netfilter package. Packets logged are\n"
            "displayed in a popup window."),
        "http://home.freeuk.net/nick.battle",
        "nick.battle@freeuk.com");

    aboutData->addAuthor("Nick Battle",
	 "Original author", "nick.battle@freeuk.com");
    aboutData->addAuthor("John Stamp",
	 "Enhancements after 1.7", "jstamp@users.sourceforge.net"); 

    KHelpMenu *help = new KHelpMenu(this, aboutData);
    KPopupMenu *helpPopup = help->menu();
    menubar->insertItem("&Help", (QPopupMenu *)helpPopup);

    resize(minimumSizeHint());	// force minimal size
}

KLogWatch::~KLogWatch()
{
    debug("Destroy KLogWatch\n");
}

void KLogWatch::slotHide()
{
    debug("Hiding\n");
    kloglist->markOldPackets();
    hide();
}

void KLogWatch::slotShow(bool force)
{
    debug("Showing\n");

    if (force || klogconfig->TRAYALERT == 0)
    {
	debug("Setting active focus\n");
	kloglist->setFocus();
	show();
	raise();
	setActiveWindow();
    }
}

void KLogWatch::slotSetup()
{
    bool repeat = true;
    debug("Setup\n");

    while (repeat)
    {
	KLogSetup *dialog = new KLogSetup(this);
	int choice = dialog->exec();
	delete dialog;

	if (choice == QDialog::Rejected)
	    break;

	int result = KMessageBox::warningYesNo(this,
	    "You need to restart klogwatch for your changes to take effect\n"
	    "Do you want to restart the program now?", "Restart?");

        const char *argv0 = KCmdLineArgs::appName();

	switch (result)
	{
	    case KMessageBox::Yes:
	        debug("Running: %s\n", argv0);
		execlp(argv0, argv0, NULL);
		debug("Exec failed: %s\n", strerror(errno));
		break;

	    case KMessageBox::No:
		debug("No\n");
		repeat = false;
		break;

	    default:
		break;
	}
    }
}

void KLogWatch::slotSave()
{
    debug("Saving\n");
    kloglist->thinking();

    QString filename =
	QFileDialog::getSaveFileName(QString::null, QString::null, this);

    if (!filename.isNull())
    {
	QFile file(filename);

	if (file.open(IO_WriteOnly))
	{
	    QTextStream s(&file);
	    kloglist->saveTo(s);
	    file.close();
	}
	else
	{
	    KMessageBox::sorry(this,
	    	QString("Cannot open file '%1'").arg(filename));
	}
    }
}

void KLogWatch::slotPrint()
{
    debug("Print\n");
    kloglist->thinking();
    KPrinter print;

    if (print.setup(this))
    {
        kloglist->printTo(print);
    }
}

void KLogWatch::slotExit()
{
    debug("Exiting\n");
    kapp->exit(0);
}

void KLogWatch::closeEvent( QCloseEvent *e ){
  e->ignore();
  // if the tray is visible, we don't want to close the program,
  // just hide the log window
  if (klogconfig->SYSTRAY) {
    kloglist->running();  //are we "running" or "thinking" here?
    kloglist->markOldPackets();
    hide();
  }
  //otherwise, we'll ask if we really want to close
  else {
    if ( KMessageBox::questionYesNo(this, "Do you want to close KLogwatch?")
            == KMessageBox::Yes ) {
        slotExit();
    }
  }
}

// eof
