/*
 * Klogwatch, Netfilter log monitor
 *
 * Copyright (C) 2004 Nick Battle <nick.battle@freeuk.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kuniqueapplication.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "klogwatch.h"
#include "klogsetup.h"
#include "klogpid.h"
#include "debug.h"


static KCmdLineOptions options[] =
{
    { "f <file>", "The file to monitor", 0 },
    { "k",        "Kill all running instances", 0 },
    { "p",        "Popup window of any hidden process", 0 },
    { "d",	  "Debug - no daemon mode, and show window", 0 },
    { 0, 0, 0 }
};

bool debugOn = false;	// See debug.h
KLogConfig *klogconfig = NULL;

int main(int argc, char *argv[])
{
    KCmdLineArgs::init(argc, argv, "klogwatch",
#if KDE_IS_VERSION(3,2,2)
	I18N_NOOP("KLogWatch"),
#endif
	"A simple Netfilter log file monitor", VERSION);

    KCmdLineArgs::addCmdLineOptions(options);
    KUniqueApplication::addCmdLineOptions();
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    debugOn = args->isSet("d");

    if (args->isSet("k"))
    {
        return KLogWatchKill(SIGTERM);
    }
    else if (args->isSet("p"))
    {
        return KLogWatchKill(SIGUSR1);
    }
    else
    {
	KApplication *app;

	// KUniqueApplication daemonizes the program automatically
	// so we use KApplication if debug is set.

	if (args->isSet("d"))
	{
	    app = new KApplication();
	}
	else
	{
	    app = new KUniqueApplication();
	}

        KLogSigInit();
	klogconfig = KLogConfig::getConfig();
        QCString file;
	
	if (args->isSet("f"))
	{
	    file = args->getOption("f");
	    klogconfig->FILENAME = file;
	}
	else
	{
	    file = klogconfig->FILENAME;
	}

	// Check the file is readable first, else open Setup

	while (true)
	{
	    int fd = open((const char *)file, O_RDONLY);
	    struct stat buf;

	    if ((fd >= 0) &&
		(fstat(fd, &buf) >= 0) &&
		(S_ISREG(buf.st_mode)))
	    {
		debug("Logfile %s OK\n", (const char *)file);
		break;
	    }

            KMessageBox::sorry(0,
	    	QString("Cannot open Netfilter log file '%1'\n"
		    "File must be readable and not a directory\n"
		    "Please update setup filename or use -f...").arg(file));

	    KLogSetup *dialog = new KLogSetup(NULL);
	    int choice = dialog->exec();
	    delete dialog;

	    if (choice == QDialog::Rejected)
	    {
		debug("Give up\n");
		exit(1);
	    }

	    file = klogconfig->FILENAME;
	}

        QWidget *d = QApplication::desktop();
        int screenh = d->height();
        int screenw = d->width();
        debug("Screen = %dx%d\n", screenw, screenh);
        int rv = 0;

        try
        {
            KLogWatch *main = new KLogWatch(file);

	    // This actions the resizes so that the code that
	    // follows can ask for the width/height and get the
	    // right answers, even though the widget has not 
	    // been shown.
	    QApplication::sendPostedEvents();

            main->move((screenw - main->width())/2,
                       (screenh - main->height())/2);

	    if (args->isSet("d"))
	    {
		main->show();
	    }

            rv = app->exec();
        }
        catch (QString ex)
        {
            KMessageBox::sorry(0, QString("%1: %2").arg(file).arg(ex));
            rv = 1;
        }
        catch (...)
        {
            KMessageBox::sorry(0, QString("Cannot open file '%1'").arg(file));
            rv = 1;
        }

        return rv;
    }

    return 0;
}

// eof
